<!--

@license Apache-2.0

Copyright (c) 2023 The Stdlib Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-->


<details>
  <summary>
    About stdlib...
  </summary>
  <p>We believe in a future in which the web is a preferred environment for numerical computation. To help realize this future, we've built stdlib. stdlib is a standard library, with an emphasis on numerical and scientific computation, written in JavaScript (and C) for execution in browsers and in Node.js.</p>
  <p>The library is fully decomposable, being architected in such a way that you can swap out and mix and match APIs and functionality to cater to your exact preferences and use cases.</p>
  <p>When you use stdlib, you can be absolutely certain that you are using the most thorough, rigorous, well-written, studied, documented, tested, measured, and high-quality code out there.</p>
  <p>To join us in bringing numerical computing to the web, get started by checking us out on <a href="https://github.com/stdlib-js/stdlib">GitHub</a>, and please consider <a href="https://opencollective.com/stdlib">financially supporting stdlib</a>. We greatly appreciate your continued support!</p>
</details>

# Unary

[![NPM version][npm-image]][npm-url] [![Build Status][test-image]][test-url] [![Coverage Status][coverage-image]][coverage-url] <!-- [![dependencies][dependencies-image]][dependencies-url] -->

> Constructor for creating arrays filled with pseudorandom values drawn from a unary PRNG.

<section class="installation">

## Installation

```bash
npm install @stdlib/random-array-tools-unary
```

</section>

<section class="usage">

## Usage

```javascript
var Random = require( '@stdlib/random-array-tools-unary' );
```

#### Random( prng, dtypes, dtype )

Constructor for creating arrays filled with pseudorandom values drawn from a unary PRNG.

```javascript
var exponential = require( '@stdlib/random-base-exponential' );

var dtypes = [ 'float64', 'float32', 'generic' ];
var defaultDType = 'float64';

var random = new Random( exponential, dtypes, defaultDType );
```

The constructor has the following parameters:

-   **prng**: unary pseudorandom value generator.
-   **dtypes**: list of supported output data types.
-   **dtype**: default output data type.

#### Random.prototype.generate( len, param1\[, options] )

Returns an array filled with pseudorandom values drawn from a unary PRNG.

```javascript
var exponential = require( '@stdlib/random-base-exponential' );

var dtypes = [ 'float64', 'float32', 'generic' ];
var defaultDType = 'float64';

var random = new Random( exponential, dtypes, defaultDType );

var v = random.generate( 10, 2.0 );
// returns <Float64Array>
```

The method has the following parameters:

-   **len**: output array length.
-   **param1**: PRNG parameter.
-   **options**: function options.

The method accepts the following options:

-   **dtype**: output array data type. Setting this option, overrides the default output array data type.

By default, the method returns an array having the default output array data type. To override the default, set the `dtype` option.

```javascript
var exponential = require( '@stdlib/random-base-exponential' );

var dtypes = [ 'float64', 'float32', 'generic' ];
var defaultDType = 'float64';

var random = new Random( exponential, dtypes, defaultDType );

var v = random.generate( 10, 2.0, {
    'dtype': 'float32'
});
// returns <Float32Array>
```

#### Random.prototype.assign( param1, out )

Fills an array with pseudorandom values drawn from a unary PRNG.

```javascript
var exponential = require( '@stdlib/random-base-exponential' );
var zeros = require( '@stdlib/array-zeros' );

var dtypes = [ 'float64', 'float32', 'generic' ];
var defaultDType = 'float64';

var random = new Random( exponential, dtypes, defaultDType );

var out = zeros( 10, 'float64' );
// returns <Float64Array>

var v = random.assign( 2.0, out );
// returns <Float64Array>

var bool = ( v === out );
// returns true
```

The method has the following parameters:

-   **param1**: PRNG parameter.
-   **out**: output array.

</section>

<!-- /.usage -->

<section class="notes">

</section>

<!-- /.notes -->

<section class="examples">

## Examples

<!-- eslint no-undef: "error" -->

```javascript
var exponential = require( '@stdlib/random-base-exponential' );
var dtypes = require( '@stdlib/array-dtypes' );
var Random = require( '@stdlib/random-array-tools-unary' );

var dt = dtypes( 'real_floating_point_and_generic' );
var random = new Random( exponential, dt, 'float64' );

var x = random.generate( 10, 2.0 );
// returns <Float64Array>

x = random.generate( 10, 2.0, {
    'dtype': 'float32'
});
// returns <Float32Array>

x = random.generate( 10, 2.0, {
    'dtype': 'generic'
});
// returns [...]
```

</section>

<!-- /.examples -->

<!-- Section for related `stdlib` packages. Do not manually edit this section, as it is automatically populated. -->

<section class="related">

</section>

<!-- /.related -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->


<section class="main-repo" >

* * *

## Notice

This package is part of [stdlib][stdlib], a standard library for JavaScript and Node.js, with an emphasis on numerical and scientific computing. The library provides a collection of robust, high performance libraries for mathematics, statistics, streams, utilities, and more.

For more information on the project, filing bug reports and feature requests, and guidance on how to develop [stdlib][stdlib], see the main project [repository][stdlib].

#### Community

[![Chat][chat-image]][chat-url]

---

## License

See [LICENSE][stdlib-license].


## Copyright

Copyright &copy; 2016-2024. The Stdlib [Authors][stdlib-authors].

</section>

<!-- /.stdlib -->

<!-- Section for all links. Make sure to keep an empty line after the `section` element and another before the `/section` close. -->

<section class="links">

[npm-image]: http://img.shields.io/npm/v/@stdlib/random-array-tools-unary.svg
[npm-url]: https://npmjs.org/package/@stdlib/random-array-tools-unary

[test-image]: https://github.com/stdlib-js/random-array-tools-unary/actions/workflows/test.yml/badge.svg?branch=v0.2.0
[test-url]: https://github.com/stdlib-js/random-array-tools-unary/actions/workflows/test.yml?query=branch:v0.2.0

[coverage-image]: https://img.shields.io/codecov/c/github/stdlib-js/random-array-tools-unary/main.svg
[coverage-url]: https://codecov.io/github/stdlib-js/random-array-tools-unary?branch=main

<!--

[dependencies-image]: https://img.shields.io/david/stdlib-js/random-array-tools-unary.svg
[dependencies-url]: https://david-dm.org/stdlib-js/random-array-tools-unary/main

-->

[chat-image]: https://img.shields.io/gitter/room/stdlib-js/stdlib.svg
[chat-url]: https://app.gitter.im/#/room/#stdlib-js_stdlib:gitter.im

[stdlib]: https://github.com/stdlib-js/stdlib

[stdlib-authors]: https://github.com/stdlib-js/stdlib/graphs/contributors

[umd]: https://github.com/umdjs/umd
[es-module]: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules

[deno-url]: https://github.com/stdlib-js/random-array-tools-unary/tree/deno
[deno-readme]: https://github.com/stdlib-js/random-array-tools-unary/blob/deno/README.md
[umd-url]: https://github.com/stdlib-js/random-array-tools-unary/tree/umd
[umd-readme]: https://github.com/stdlib-js/random-array-tools-unary/blob/umd/README.md
[esm-url]: https://github.com/stdlib-js/random-array-tools-unary/tree/esm
[esm-readme]: https://github.com/stdlib-js/random-array-tools-unary/blob/esm/README.md
[branches-url]: https://github.com/stdlib-js/random-array-tools-unary/blob/main/branches.md

[stdlib-license]: https://raw.githubusercontent.com/stdlib-js/random-array-tools-unary/main/LICENSE

</section>

<!-- /.links -->
